package com.ruoyi.project.document.mapper;

import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.common.domain.DataEntity;
import com.ruoyi.project.document.domain.TPlantproglist;

import java.util.List;
import java.util.Map;

/**
 * 装置程序清单Mapper接口
 *
 * @author ruoyi
 * @date 2020-11-26
 */
public interface TPlantproglistMapper
{
    /**
     * 查询装置程序清单
     *
     * @param id 装置程序清单ID
     * @return 装置程序清单
     */
    public TPlantproglist selectTPlantproglistById(Long id);

    /**
     * 查询装置程序清单列表
     *
     * @param tPlantproglist 装置程序清单
     * @return 装置程序清单集合
     */
    @DataScope(deptAlias = "d")
    public List<TPlantproglist> selectTPlantproglistList(TPlantproglist tPlantproglist);

    public List<TPlantproglist> countNextReviewDate(TPlantproglist tPlantproglist);

    public List<TPlantproglist> countNextEditDate(TPlantproglist tPlantproglist);

    /**
     * 定时任务查询装置程序清单列表
     *
     * @param tPlantproglist 装置程序清单
     * @return 装置程序清单集合
     */
    public List<TPlantproglist> selectList(TPlantproglist tPlantproglist);

    /**
     * 新增装置程序清单
     *
     * @param tPlantproglist 装置程序清单
     * @return 结果
     */
    public int insertTPlantproglist(TPlantproglist tPlantproglist);

    /**
     * 修改装置程序清单
     *
     * @param tPlantproglist 装置程序清单
     * @return 结果
     */
    public int updateTPlantproglist(TPlantproglist tPlantproglist);

    /**
     * 删除装置程序清单
     *
     * @param id 装置程序清单ID
     * @return 结果
     */
    public int deleteTPlantproglistById(Long id);

    /**
     * 批量删除装置程序清单
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTPlantproglistByIds(Long[] ids);

    List<DataEntity> selectRcData(Map param);

    List<DataEntity> selectclassifyData(Map param);
}
